import time

from ..base import ValueInterface
from ..base import ChangedValueEntity
from ..base import AbstractItem, AbstractProperty


class TimePointTypeMixin(ValueInterface):
    @property
    def value(self) -> str:
        return self.get_value()

    @value.setter
    def value(self, value):
        self.set_value(value)


class TimePointTypeProperty(AbstractProperty, TimePointTypeMixin):
    def update(self, t_format='%Y-%m-%d %H:%M:%S'):
        self.value = time.strftime(t_format)


class TimePointPropertyInterface(ChangedValueEntity):
    @property
    def time(self) -> TimePointTypeProperty:
        return self.attach(creator=TimePointTypeProperty, args=(
            self.get_time_value, self.set_time_value, self.changed
        ))

    def get_time_value(self) -> str:
        return self.get_value()

    def set_time_value(self, value: str):
        return self.set_value(value)


class TimePointItem(AbstractItem, TimePointPropertyInterface):
    pass



from PySide2.QtWidgets import QInputDialog


class InputDialog(QInputDialog):  # pragma: no cover
    @staticmethod
    def get_double(parent=None, title='Input Double Value', label='Value:',
                   default_value=None, decimals=1, minimum=1, maximum=1e9):
        input_dialog = QInputDialog(parent)
        input_dialog.setInputMode(InputDialog.DoubleInput)
        input_dialog.setWindowTitle(title)
        input_dialog.setLabelText(label)
        input_dialog.setOkButtonText('确定')
        input_dialog.setCancelButtonText('取消')
        input_dialog.setDoubleDecimals(decimals)
        input_dialog.setDoubleRange(minimum, maximum)
        input_dialog.setDoubleValue(default_value or minimum)
        if parent:
            input_dialog.setFont(parent.font())

        return default_value if not input_dialog.exec_() else input_dialog.doubleValue()

    @staticmethod
    def get_int(parent=None, title='Input Int Value', label='Value:',
                default_value=None, minimum=1, maximum=100):
        input_dialog = QInputDialog(parent)
        input_dialog.setInputMode(InputDialog.IntInput)
        input_dialog.setWindowTitle(title)
        input_dialog.setLabelText(label)
        input_dialog.setOkButtonText('确定')
        input_dialog.setCancelButtonText('取消')
        input_dialog.setIntRange(minimum, maximum)
        input_dialog.setIntValue(default_value or minimum)
        if parent:
            input_dialog.setFont(parent.font())

        return default_value if not input_dialog.exec_() else input_dialog.intValue()

    @staticmethod
    def get_text(parent=None, title='Input Text Value', label='Value:', default_value=None):
        input_dialog = QInputDialog(parent)
        input_dialog.setInputMode(InputDialog.TextInput)
        input_dialog.setWindowTitle(title)
        input_dialog.setLabelText(label)
        input_dialog.setOkButtonText('确定')
        input_dialog.setCancelButtonText('取消')
        if parent:
            input_dialog.setFont(parent.font())

        return default_value if not input_dialog.exec_() else input_dialog.textValue()



from . import BaseWidget
from ...model import WidgetStringItem
from PySide2.QtWidgets import QDateEdit
from PySide2.QtCore import QDate


class DateEdit(QDateEdit, BaseWidget, WidgetStringItem):
    def get_value(self) -> str:
        return self.text()

    def set_value(self, value: str):
        value = value or ''
        if value != self.get_value():
            date = value.split('-')
            if len(date) != 3:
                raise ValueError('Date format is invalid')
            self.setDate(QDate(int(date[0]), int(date[1]), int(date[2])))

    def set_changed_connection(self):
        # noinspection PyUnresolvedReferences
        self.dateChanged.connect(self.check_change)
